package build

import (
	"sync"
)

// Compile compiles all the functions.
func Compile(functions <-chan *Function, errors <-chan error) (map[string]*Function, error) {
	allFunctions := map[string]*Function{}

	for functions != nil || errors != nil {
		select {
		case err, ok := <-errors:
			if !ok {
				errors = nil
				continue
			}

			return nil, err

		case function, ok := <-functions:
			if !ok {
				functions = nil
				continue
			}

			allFunctions[function.Name] = function
		}
	}

	wg := sync.WaitGroup{}

	for _, function := range allFunctions {
		wg.Add(1)

		go func() {
			defer wg.Done()
			function.Compile()
		}()
	}

	wg.Wait()

	for _, function := range allFunctions {
		if function.Error != nil {
			return nil, function.Error
		}
	}

	return allFunctions, nil
}
