package elf

import (
	"encoding/binary"
	"io"

	"git.akyoto.dev/cli/q/src/build/config"
)

// ELF represents an ELF file.
type ELF struct {
	Header
	ProgramHeader
	Code []byte
	Data []byte
}

// New creates a new ELF binary.
func New(code []byte, data []byte) *ELF {
	elf := &ELF{
		Header: Header{
			Magic:                       [4]byte{0x7F, 'E', 'L', 'F'},
			Class:                       2,
			Endianness:                  LittleEndian,
			Version:                     1,
			OSABI:                       0,
			ABIVersion:                  0,
			Type:                        TypeExecutable,
			Architecture:                ArchitectureAMD64,
			FileVersion:                 1,
			EntryPointInMemory:          config.BaseAddress + config.CodeOffset,
			ProgramHeaderOffset:         HeaderSize,
			SectionHeaderOffset:         0,
			Flags:                       0,
			Size:                        HeaderSize,
			ProgramHeaderEntrySize:      ProgramHeaderSize,
			ProgramHeaderEntryCount:     1,
			SectionHeaderEntrySize:      SectionHeaderSize,
			SectionHeaderEntryCount:     0,
			SectionNameStringTableIndex: 0,
		},
		ProgramHeader: ProgramHeader{
			Type:            ProgramTypeLOAD,
			Flags:           ProgramFlagsExecutable,
			Offset:          config.CodeOffset,
			VirtualAddress:  config.BaseAddress + config.CodeOffset,
			PhysicalAddress: config.BaseAddress + config.CodeOffset,
			SizeInFile:      int64(len(code)),
			SizeInMemory:    int64(len(code)),
			Align:           config.Align,
		},
		Code: code,
		Data: data,
	}

	return elf
}

// Write writes the ELF64 format to the given writer.
func (elf *ELF) Write(writer io.Writer) {
	binary.Write(writer, binary.LittleEndian, &elf.Header)
	binary.Write(writer, binary.LittleEndian, &elf.ProgramHeader)
	writer.Write([]byte{0, 0, 0, 0, 0, 0, 0, 0})
	writer.Write(elf.Code)
	writer.Write(elf.Data)
}
